@@ -1,6 +1,6 @@
 /*
  * r3_define.h
- * Copyright (C) 2014 c9s <c9s@c9smba.local>
+ * Copyright (C) 2014 c9s <yoanlin93@gmail.com>
  *
  * Distributed under terms of the MIT license.
  */
@@ -9,7 +9,7 @@
 #define DEFINE_H
 #include <stdbool.h>
 
-#ifndef bool
+#if !defined(bool) && !defined(__cplusplus)
 typedef unsigned char bool;
 #endif
 #ifndef FALSE
@@ -23,7 +23,7 @@ typedef unsigned char bool;
 #ifdef DEBUG
 
 #define info(fmt, ...) \
-            do { fprintf(stderr, fmt, __VA_ARGS__); } while (0)
+            do { fprintf(stderr, fmt, ##__VA_ARGS__); } while (0)
 
 #define debug(fmt, ...) \
         do { fprintf(stderr, "%s:%d:%s(): " fmt, __FILE__, \
