@@ -60,6 +60,7 @@ node * r3_tree_create(int cap) {
     n->combined_pattern = NULL;
     n->pcre_pattern = NULL;
     n->pcre_extra = NULL;
+    n->data = NULL;
     return n;
 }
 
@@ -71,7 +72,6 @@ void r3_tree_free(node * tree) {
     }
     zfree(tree->edges);
     zfree(tree->routes);
-
     if (tree->pcre_pattern) {
         pcre_free(tree->pcre_pattern);
     }
@@ -173,26 +173,24 @@ int r3_tree_compile(node *n, char **errstr)
  * Return 0 if success
  */
 int r3_tree_compile_patterns(node * n, char **errstr) {
-    char * cpat;
+    edge * e = NULL;
     char * p;
-
-    cpat = zcalloc(sizeof(char) * 220); // XXX
+    char * cpat = zcalloc(sizeof(char) * 64 * 3); // XXX
     if (!cpat) {
         asprintf(errstr, "Can not allocate memory");
         return -1;
     }
 
     p = cpat;
-
-    edge *e = NULL;
-    int opcode_cnt =  0;
-    for ( int i = 0 ; i < n->edge_len ; i++ ) {
+    int opcode_cnt = 0;
+    int i = 0;
+    for (; i < n->edge_len ; i++) {
         e = n->edges[i];
-
-        if ( e->opcode )
+        if (e->opcode) {
             opcode_cnt++;
+        }
 
-        if ( e->has_slug ) {
+        if (e->has_slug) {
             // compile "foo/{slug}" to "foo/[^/]+"
             char * slug_pat = r3_slug_compile(e->pattern, e->pattern_len);
             strcat(p, slug_pat);
@@ -277,14 +275,17 @@ node * r3_tree_matchl(const node * n, const char * path, int path_len, match_ent
     info("try matching: %s\n", path);
 
     edge *e;
-    unsigned short i;
-    unsigned short restlen;
+    unsigned int i;
+    unsigned int restlen;
+
+    const char *pp;
+    const char *pp_end;
 
     if (n->compare_type == NODE_COMPARE_OPCODE) {
-        char *pp;
-        const char *pp_end = path + path_len;
-        for (i = 0; i < n->edge_len ; i++ ) {
-            pp = (char*) path;
+        pp_end = path + path_len;
+
+        for (i = n->edge_len; i--; ) {
+            pp = path;
             e = n->edges[i];
             switch(e->opcode) {
                 case OP_EXPECT_NOSLASH:
@@ -305,14 +306,14 @@ node * r3_tree_matchl(const node * n, const char * path, int path_len, match_ent
             }
             // check match
             if ( (pp - path) > 0) {
-                restlen = pp_end - pp;
                 if (entry) {
                     str_array_append(entry->vars , zstrndup(path, pp - path));
                 }
+                restlen = pp_end - pp;
                 if (restlen == 0) {
                     return e->child && e->child->endpoint > 0 ? e->child : NULL;
                 }
-                return r3_tree_matchl(e->child, pp, pp_end - pp, entry);
+                return r3_tree_matchl(e->child, pp, restlen, entry);
             }
         }
     }
@@ -320,10 +321,10 @@ node * r3_tree_matchl(const node * n, const char * path, int path_len, match_ent
     // if the pcre_pattern is found, and the pointer is not NULL, then it's
     // pcre pattern node, we use pcre_exec to match the nodes
     if (n->pcre_pattern) {
-        char *substring_start = NULL;
+        const char *substring_start = NULL;
         int   substring_length = 0;
         int   ov[ n->ov_cnt ];
-        char rc;
+        int   rc;
 
         info("pcre matching %s on %s\n", n->combined_pattern, path);
 
@@ -357,29 +358,54 @@ node * r3_tree_matchl(const node * n, const char * path, int path_len, match_ent
         }
 
 
-        for (i = 1; i < rc; i++)
-        {
-            substring_start = ((char*) path) + ov[2*i];
-            substring_length = ov[2*i+1] - ov[2*i];
-            // info("%2d: %.*s\n", i, substring_length, substring_start);
 
-            if ( substring_length > 0) {
-                restlen = path_len - ov[1]; // fully match to the end
-                // info("matched item => restlen:%d edges:%d i:%d\n", restlen, n->edge_len, i);
+        restlen = path_len - ov[1]; // if it's fully matched to the end (rest string length)
 
+        if (restlen == 0 ) {
+            // Check the substring to decide we should go deeper on which edge 
+            for (i = 1; i < rc; i++)
+            {
+                substring_length = ov[2*i+1] - ov[2*i];
+
+                // if it's not matched for this edge, just skip them quickly
+                if (substring_length == 0)
+                    continue;
+
+                substring_start = path + ov[2*i];
                 e = n->edges[i - 1];
 
                 if (entry && e->has_slug) {
                     // append captured token to entry
                     str_array_append(entry->vars , zstrndup(substring_start, substring_length));
                 }
-                if (restlen == 0 ) {
-                    return e->child && e->child->endpoint > 0 ? e->child : NULL;
-                }
-                // get the length of orginal string: $0
-                return r3_tree_matchl( e->child, path + (ov[1] - ov[0]), restlen, entry);
+
+                // since restlen == 0 return the edge quickly.
+                return e->child && e->child->endpoint > 0 ? e->child : NULL;
             }
         }
+
+
+        // Check the substring to decide we should go deeper on which edge 
+        for (i = 1; i < rc; i++)
+        {
+            substring_length = ov[2*i+1] - ov[2*i];
+
+            // if it's not matched for this edge, just skip them quickly
+            if ( substring_length == 0) {
+                continue;
+            }
+
+            substring_start = path + ov[2*i];
+            e = n->edges[i - 1];
+
+            if (entry && e->has_slug) {
+                // append captured token to entry
+                str_array_append(entry->vars , zstrndup(substring_start, substring_length));
+            }
+
+            // get the length of orginal string: $0
+            return r3_tree_matchl( e->child, path + (ov[1] - ov[0]), restlen, entry);
+        }
         // does not match
         return NULL;
     }
@@ -398,10 +424,10 @@ node * r3_tree_matchl(const node * n, const char * path, int path_len, match_ent
 
 route * r3_tree_match_route(const node *tree, match_entry * entry) {
     node *n;
+    int i;
     n = r3_tree_match_entry(tree, entry);
     if (n && n->routes && n->route_len > 0) {
-        int i;
-        for (i = 0; i < n->route_len ; i++ ) {
+        for (i = n->route_len; i--; ) {
             if ( r3_route_cmp(n->routes[i], entry) == 0 ) {
                 return n->routes[i];
             }
@@ -411,12 +437,12 @@ route * r3_tree_match_route(const node *tree, match_entry * entry) {
 }
 
 inline edge * r3_node_find_edge_str(const node * n, const char * str, int str_len) {
-    unsigned short i = 0;
     char firstbyte = *str;
-    for (; i < n->edge_len ; i++ ) {
-        if ( firstbyte == *(n->edges[i]->pattern) ) {
-            info("matching '%s' with '%s'\n", str, node_edge_pattern(n,i) );
-            if ( strncmp( node_edge_pattern(n,i), str, node_edge_pattern_len(n,i) ) == 0 ) {
+    unsigned int i;
+    for (i = n->edge_len; i--; ) {
+        edge *e = n->edges[i];
+        if (firstbyte == e->pattern[0]) {
+            if (strncmp(e->pattern, str, e->pattern_len) == 0 ) {
                 return n->edges[i];
             }
             return NULL;
@@ -440,14 +466,10 @@ node * r3_node_create() {
     n->combined_pattern = NULL;
     n->pcre_pattern = NULL;
     n->pcre_extra = NULL;
+    n->data = NULL;
     return n;
 }
 
-
-route * r3_route_create(const char * path) {
-    return r3_route_createl(path, strlen(path));
-}
-
 void r3_route_free(route * route) {
     zfree(route);
 }
@@ -490,12 +512,6 @@ route * r3_tree_insert_routel_ex(node *tree, int method, const char *path, int p
 
 
 
-node * r3_tree_insert_pathl(node *tree, const char *path, int path_len, void * data)
-{
-    return r3_tree_insert_pathl_ex(tree, path, path_len, NULL , data, NULL);
-}
-
-
 /**
  * Find common prefix from the edges of the node.
  *
@@ -551,12 +567,15 @@ edge * r3_node_find_common_prefix(node *n, const char *path, int path_len, int *
                     break;
                 }
             } else if (ret == -1) {
-                
+                r3_slug_free(slug);
                 return NULL;
             } else {
                 break;
             }
         } while(ret == 1);
+
+        // free the slug
+        r3_slug_free(slug);
     }
 
     *prefix_len = prefix;
@@ -573,10 +592,19 @@ node * r3_tree_insert_pathl_ex(node *tree, const char *path, int path_len, route
 {
     node * n = tree;
 
-
     // common edge
     edge * e = NULL;
 
+    // If there is no path to insert at the node, we just increase the mount
+    // point on the node and append the route.
+    if (path_len == 0) {
+        tree->endpoint++;
+        if (route) {
+            route->data = data;
+            r3_node_append_route(tree, route);
+        }
+        return tree;
+    }
 
     /* length of common prefix */
     int prefix_len = 0;
@@ -594,7 +622,7 @@ node * r3_tree_insert_pathl_ex(node *tree, const char *path, int path_len, route
     // common prefix not found, insert a new edge for this pattern
     if ( prefix_len == 0 ) {
         // there are two more slugs, we should break them into several parts
-        int slug_cnt = slug_count(path, path_len, errstr);
+        int slug_cnt = r3_slug_count(path, path_len, errstr);
         if (slug_cnt == -1) {
             return NULL;
         }
@@ -697,17 +725,23 @@ node * r3_tree_insert_pathl_ex(node *tree, const char *path, int path_len, route
         } else {
             // there are no more path to insert
 
-            // see if there is an endpoint already
-            if (e->child->endpoint > 0) {
-                // XXX: return an error code instead of NULL
-                return NULL;
-            }
-            e->child->endpoint++; // make it as an endpoint
-            e->child->data = data;
+            // see if there is an endpoint already, we should n't overwrite the data on child.
+            // but we still need to append the route.
+
             if (route) {
                 route->data = data;
                 r3_node_append_route(e->child, route);
+                e->child->endpoint++; // make it as an endpoint
+                return e->child;
             }
+
+            // insertion without route
+            if (e->child->endpoint > 0) {
+                // TODO: return an error code instead of NULL
+                return NULL;
+            }
+            e->child->endpoint++; // make it as an endpoint
+            e->child->data = data; // set data
             return e->child;
         }
 
@@ -777,19 +811,13 @@ void r3_tree_dump(const node * n, int level) {
  *
  * -1 == different route
  */
-int r3_route_cmp(const route *r1, const match_entry *r2) {
+inline int r3_route_cmp(const route *r1, const match_entry *r2) {
     if (r1->request_method != 0) {
         if (0 == (r1->request_method & r2->request_method) ) {
             return -1;
         }
     }
 
-    if ( r1->path && r2->path ) {
-        if ( strcmp(r1->path, r2->path) != 0 ) {
-            return -1;
-        }
-    }
-
     if ( r1->host && r2->host ) {
         if (strcmp(r1->host, r2->host) != 0 ) {
             return -1;
